export default class AmpCafAdPlugin extends akamai.amp.AdPlugin {

  /**
   *
   */
  constructor(player, config, key) {
    super(player, config, key)
    this.feature = "ads"

    this.bindHandlers(["onLoad", "onBreakStarted", "onBreakEnded", "onBreakClipStarted", "onBreakClipEnded"])

    const manager = this.playerManager
    manager.setMessageInterceptor(cast.framework.messages.MessageType.LOAD, this.onLoad)
    manager.addEventListener(cast.framework.events.EventType.BREAK_STARTED, this.onBreakStarted)
    manager.addEventListener(cast.framework.events.EventType.BREAK_ENDED, this.onBreakEnded)
    manager.addEventListener(cast.framework.events.EventType.BREAK_CLIP_STARTED, this.onBreakClipStarted)
    manager.addEventListener(cast.framework.events.EventType.BREAK_CLIP_ENDED, this.onBreakClipEnded)
  }

  get playerManager() {
    return this.player.playerManager
  }

  get breakManager() {
    return this.playerManager.getBreakManager()
  }

  onLoad(loadRequestData) {
    loadRequestData.media.vmapAdsRequest = {
      adTagUrl: this.data.vmapAdsRequest
    }
    return loadRequestData
  }

  onBreakStarted(event) {
    const adBreak = this.breakManager.getBreakById(event.breakId)
    this.currentBreak = akamai.amp.Ad.create(
      adBreak.id,             // id
      null,                   // title
      adBreak.duration,       // duration
      adBreak.position,       // position
      adBreak.id,             // type
      this.data.provider,     // partner
      null,                   // advertiser
      null,                   // companions
      null,                   // request
      event,                  // metadata
      event.total,            // totalAds
      event.currentMediaTime, // time
      null                    // clickThrough
    )
    this.dispatch(akamai.amp.AdEvents.BREAK_START, this.currentBreak)
  }

  onBreakEnded(event) {
    this.dispatch(akamai.amp.AdEvents.BREAK_END, this.currentBreak)
  }

  onBreakClipStarted(event) {
    const clip = this.breakManager.getBreakClipById(event.breakClipId)
    this.currentAd = akamai.amp.Ad.create(
      clip.id,                // id
      clip.title,             // title
      clip.duration,          // duration
      event.index,            // position
      event.breakId,          // type
      this.data.provider,     // partner
      null,                   // advertiser
      null,                   // companions
      null,                   // request
      event,                  // metadata
      event.total,            // totalAds
      event.currentMediaTime, // time
      clip.clickThroughUrl    // clickThrough
    )
    this.dispatch(akamai.amp.AdEvents.AD_STARTED, this.currentAd)
  }

  onBreakClipEnded(event) {
    this.dispatch(akamai.amp.AdEvents.AD_ENDED, this.currentAd)
  }
}

akamai.amp.AMP.registerPlugin("cafads", akamai.amp.Plugin.createFactory(AmpCafAdPlugin))
