(function (exports) {
'use strict';

var YoSpace = function (_akamai$amp$AdPlugin) {
  babelHelpers.inherits(YoSpace, _akamai$amp$AdPlugin);

  function YoSpace(player, config) {
    babelHelpers.classCallCheck(this, YoSpace);

    var _this = babelHelpers.possibleConstructorReturn(this, (YoSpace.__proto__ || Object.getPrototypeOf(YoSpace)).call(this, player, config));

    _this.inProgress = false;

    _this.bindHandlers(["AdBreakStart", "AdBreakEnd", "AdvertStart", "AdvertEnd", "UpdateTimeline", "AnalyticsFired", "onclick", "onadtimeupdate"]);

    YSSessionManager.DEFAULTS.DEBUGGING = _this.debug;
    _this.sdk = { YSSessionManager: YSSessionManager, YSPlayerEvents: YSPlayerEvents

      // setup transforms
    };if (config.transform !== false) {
      _this.player.addTransform(akamai.amp.TransformType.MEDIA, _this.mediaTransform.bind(_this));
    }
    return _this;
  }

  babelHelpers.createClass(YoSpace, [{
    key: "onready",
    value: function onready(event) {
      var _this2 = this;

      // TODO: This is a hack to get around the issue that hlsjs does not convert YoSpace ID3 tags into TextTrack DataCues
      if (this.player.hls) {
        this.player.hls.addEventListener("hlsFragParsingMetadata", function (event) {
          var data = event.detail;

          if (!data) return;

          var textTracks = _this2.player.textTracks;
          var textTrack = null;
          for (var i = 0, len = textTracks.length; i < len; i++) {
            if (textTracks[i].kind === "metadata") {
              textTrack = textTracks[i];
              break;
            }
          }

          if (textTrack == null) return;

          data.samples.forEach(function (sample) {
            var info = akamai.amp.Utils.arrayBufferToString(sample.data).replace(/\n|\r|ÿ/g, "");
            var regex = /(Y[A-Z]{3})([^Y]+)/g;
            if (!regex.test(info)) return;
            info.replace(regex, function (match, key, value) {
              var cue = new VTTCue(sample.pts, data.frag.endPTS, "");
              cue.type = "org.id3";
              cue.value = {
                key: key,
                value: key + value,
                info: value
              };
              textTrack.addCue(cue);
            });
          });
        });
      }
    }
  }, {
    key: "mediaTransform",
    value: function mediaTransform(media) {
      var _this3 = this;

      this.player.busy = true;
      if (this.sessionManager != null) {
        this.sessionManager.shutdown();
      }
      return this.createSession(media, this.data.props).then(function (url) {
        _this3.player.busy = false;
        _this3.logger.log("[AMP YoSpace] Session Created", url);
        media.src = url;
        return media;
      });
    }
  }, {
    key: "createSession",
    value: function createSession(media) {
      var _this4 = this;

      var props = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

      return new Promise(function (resolve, reject) {
        var temporalType = media.temporalType;
        var func = temporalType == "live" || temporalType == "dvr" ? "createForLive" : "createForVoD";
        _this4.sessionManager = YSSessionManager[func](media.src, props, function (state, result) {
          if (state !== YSSessionResult.INITIALISED) {
            var error = akamai.amp.AMPError.create("Could not create session: " + result, result, result);
            return reject(error);
          }

          if (_this4.sessionManager.isYospaceStream()) {
            _this4.sessionManager.registerPlayer(_this4);
            resolve(_this4.sessionManager.masterPlaylist());
          } else {
            _this4.sessionManager.shutdown();
            _this4.sessionManager = null;
            resolve(media.src);
          }
        });
      });
    }
  }, {
    key: "reportEvent",
    value: function reportEvent(event, detail) {
      if (this.sessionManager == null) return;
      this.sessionManager.reportPlayerEvent(event, detail);
    }
  }, {
    key: "AdBreakStart",
    value: function AdBreakStart(adBreak) {
      this.player.adContainer.addEventListener("click", this.onclick);

      this.inProgress = true;
      var adverts = adBreak.adverts;
      this.currentBreak = akamai.amp.Ad.create(null, null, adBreak.getDuration(), null, null, "yospace", null, null, null, adBreak, adverts.length, 0);
      this.dispatch(akamai.amp.AdEvents.BREAK_START, this.currentBreak);
    }
  }, {
    key: "onadtimeupdate",
    value: function onadtimeupdate(event) {
      var duration = this.currentAd.duration;
      var time = akamai.amp.Utils.clamp(event.target.currentTime - this.startTime, 0, duration);
      this.dispatch(akamai.amp.AdEvents.TIME_UPDATE, time);
      this.dispatch(akamai.amp.AdEvents.TIME_REMAINING, duration - time);
    }
  }, {
    key: "AdvertStart",
    value: function AdvertStart(data) {
      var session = this.sessionManager.session;
      var advert = session.currentAdvert;
      var adBreak = advert.adBreak;
      var meta = advert.advert;
      this.currentAd = akamai.amp.Ad.create(meta.id, // id
      meta.AdTitle, // title
      advert.duration, // duration
      meta.sequence, // position
      "midroll", // type
      "yospace", // partner
      meta.Advertiser, // advertiser
      meta.nonLinears, // companions
      session, // request
      advert, // metadata
      adBreak.adverts.length, // totalAds
      0, // time
      session.getLinearClickthrough() // clickThrough
      );
      this.dispatch(akamai.amp.AdEvents.STARTED, this.currentAd);

      var element = this.player.mediaElement;
      this.startTime = element.currentTime;
      element.addEventListener("timeupdate", this.onadtimeupdate);
    }
  }, {
    key: "AdvertEnd",
    value: function AdvertEnd(data) {
      this.player.mediaElement.removeEventListener("timeupdate", this.onadtimeupdate);
      this.dispatch(akamai.amp.AdEvents.ENDED, this.currentAd);
    }
  }, {
    key: "AdBreakEnd",
    value: function AdBreakEnd(adBreak) {
      this.player.adContainer.removeEventListener("click", this.onclick);

      this.inProgress = false;
      this.dispatch(akamai.amp.AdEvents.BREAK_END, this.currentBreak);
    }
  }, {
    key: "UpdateTimeline",
    value: function UpdateTimeline(timeline) {
      console.log("YoSpace -> UpdateTimeline", timeline);
    }
  }, {
    key: "AnalyticsFired",
    value: function AnalyticsFired(call_id, call_data) {
      // Function gets called whenever an analytics call is made by the SDK.
    }
  }, {
    key: "ontimeupdate",
    value: function ontimeupdate(event) {
      this.reportEvent(YSPlayerEvents.POSITION, event.detail);
    }
  }, {
    key: "onfullscreenchange",
    value: function onfullscreenchange(event) {
      this.reportEvent(YSPlayerEvents.FULLSCREEN, event.detail);
    }
  }, {
    key: "onmuted",
    value: function onmuted(event) {
      this.reportEvent(YSPlayerEvents.MUTE, event.detail);
    }
  }, {
    key: "ontimedmetadata",
    value: function ontimedmetadata(event) {
      var value = event.detail.value;

      if (!/^Y/.test(value.key)) return;

      if (this.id3 == null) {
        this.id3 = {};
      }

      this.id3[value.key] = value.info;

      var keys = Object.keys(this.id3);
      var diff = ["YCSP", "YDUR", "YMID", "YSEQ", "YTYP"].filter(function (i) {
        return !keys.includes(i);
      });

      if (diff.length === 0) {
        this.reportEvent(YSPlayerEvents.METADATA, this.id3);
        this.id3 = null;
      }
    }
  }, {
    key: "onmediachange",
    value: function onmediachange(event) {
      if (!this.sessionManager || !this.sessionManager.session) {
        return;
      }

      var timeline = this.sessionManager.session.timeline;
      var elements = timeline.elements;
      if (elements.length === 0) {
        return;
      }

      var fp = akamai.fp;
      var Cue = akamai.amp.Cue;
      var xf = fp.compose(fp.filter(function (element) {
        return element.type === YSTimelineElement.ADVERT;
      }), fp.map(function (element) {
        return Cue.create(element.offset);
      }));
      var cues = fp.seq(xf, timeline.elements);
      if (cues.length === 0) {
        return;
      }

      this.player.cues = cues;
    }
  }, {
    key: "onmediasequencestarted",
    value: function onmediasequencestarted(event) {
      this.reportEvent(YSPlayerEvents.START);
    }
  }, {
    key: "onmediasequenceended",
    value: function onmediasequenceended(event) {
      this.reportEvent(YSPlayerEvents.END);
    }
  }, {
    key: "onpause",
    value: function onpause(event) {
      this.reportEvent(YSPlayerEvents.PAUSE);
    }
  }, {
    key: "onresume",
    value: function onresume(event) {
      this.reportEvent(YSPlayerEvents.RESUME);
    }
  }, {
    key: "onbufferingchange",
    value: function onbufferingchange(event) {
      this.reportEvent(event.detail ? YSPlayerEvents.STALL : YSPlayerEvents.CONTINUE);
    }
  }, {
    key: "onseeking",
    value: function onseeking(event) {
      this.reportEvent(YSPlayerEvents.SEEK_START, this.player.currentTime);
    }
  }, {
    key: "onseeked",
    value: function onseeked(event) {
      this.reportEvent(YSPlayerEvents.SEEK_END, this.player.currentTime);
    }
  }, {
    key: "onclick",
    value: function onclick() {
      var url = this.sessionManager.session.getLinearClickthrough();
      if (!url) return;
      this.reportEvent(YSPlayerEvents.CLICK);
      window.open(url, "_blank");
    }
  }, {
    key: "nonlinear",
    value: function nonlinear() {
      var idx = null;
      this.reportEvent(YSPlayerEvents.NONLINEAR, idx);
    }
  }, {
    key: "destroy",
    value: function destroy() {
      if (this.sessionManager != null) {
        this.sessionManager.shutdown();
      }
      babelHelpers.get(YoSpace.prototype.__proto__ || Object.getPrototypeOf(YoSpace.prototype), "destroy", this).call(this);
    }
  }]);
  return YoSpace;
}(akamai.amp.AdPlugin);

akamai.amp.AMP.registerPlugin("yospace", akamai.amp.Plugin.createFactory(YoSpace));

exports.YoSpace = YoSpace;

}((this.akamai.amp.yospace = this.akamai.amp.yospace || {})));
//# sourceMappingURL=Yospace.js.map
